@@ -25,6 +25,7 @@
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 # THE SOFTWARE.
 import logging
+import re
 import warnings
 from typing import Dict, Iterable, List, Optional, Tuple, Union
 
@@ -36,10 +37,7 @@ from component_contribution import (
     ZenodoSettings,
 )
 from component_contribution.linalg import LINALG
-from component_contribution.predict import (
-    CCModelParameters,
-    GibbsEnergyPredictor,
-)
+from component_contribution.predict import CCModelParameters, GibbsEnergyPredictor
 from equilibrator_cache import (
     DEFAULT_COMPOUND_CACHE_SETTINGS,
     PROTON_INCHI_KEY,
@@ -48,8 +46,12 @@ from equilibrator_cache import (
     CompoundMicrospecies,
     create_compound_cache_from_zenodo,
 )
-from equilibrator_cache.exceptions import MissingDissociationConstantsException
+from equilibrator_cache.exceptions import (
+    MissingDissociationConstantsException,
+    ParseException,
+)
 from equilibrator_cache.reaction import (
+    POSSIBLE_REACTION_ARROWS,
     create_stoichiometric_matrix_from_reactions,
 )
 
@@ -103,9 +105,7 @@ def predict_protons_and_charge(
             n_h += coeff
             z += coeff
         else:
-            ms = find_most_abundant_ms(
-                cpd, p_h, p_mg, ionic_strength, temperature
-            )
+            ms = find_most_abundant_ms(cpd, p_h, p_mg, ionic_strength, temperature)
             n_h += coeff * ms.number_protons
             n_mg += coeff * ms.number_magnesiums
             z += coeff * ms.charge
@@ -248,9 +248,7 @@ class ComponentContribution(object):
         """
         ccache = create_compound_cache_from_zenodo(ccache_settings)
         parameters = CCModelParameters.from_zenodo(cc_params_settings)
-        predictor = GibbsEnergyPredictor(
-            parameters=parameters, rmse_inf=rmse_inf
-        )
+        predictor = GibbsEnergyPredictor(parameters=parameters, rmse_inf=rmse_inf)
         return ComponentContribution(ccache=ccache, predictor=predictor)
 
     def get_compound(self, compound_id: str) -> Union[Compound, None]:
@@ -318,7 +316,7 @@ class ComponentContribution(object):
 
         Returns
         -------
-        rxn : Reaction
+        rxn : PhasedReaction
         """
         return PhasedReaction.parse_formula(self.ccache.get_compound, formula)
 
@@ -332,7 +330,7 @@ class ComponentContribution(object):
 
         Returns
         -------
-        rxn : Reaction
+        rxn : PhasedReaction
         """
         results = self.reaction_parser.parseString(formula)
         substrates, arrow, products = results
@@ -362,9 +360,7 @@ class ComponentContribution(object):
             "InChI=1S/CO2/c2-1-3",  # CO2
             "InChI=1S/H3N/h1H3/p+1",  # NH4+
         ]
-        compounds = list(
-            map(self.ccache.get_compound_by_inchi, balancing_inchis)
-        )
+        compounds = list(map(self.ccache.get_compound_by_inchi, balancing_inchis))
 
         S = self.ccache.get_element_data_frame(compounds)
         balancing_atoms = S.index
@@ -375,15 +371,12 @@ class ComponentContribution(object):
                 "Cannot balance this reaction due to missing chemical formulas"
             )
             return reaction
-        atom_vector = np.array(
-            list(map(lambda a: atom_bag.get(a, 0), balancing_atoms))
-        )
+        atom_vector = np.array(list(map(lambda a: atom_bag.get(a, 0), balancing_atoms)))
 
         other_atoms = set(atom_bag.keys()).difference(balancing_atoms)
         if other_atoms:
             raise ValueError(
-                f"Cannot oxidize {reaction} only with these atoms: "
-                f"{other_atoms}"
+                f"Cannot oxidize {reaction} only with these atoms: " f"{other_atoms}"
             )
 
         # solve the linear equation S * i = a,
@@ -530,10 +523,7 @@ class ComponentContribution(object):
             the dG_prime in kJ/mol and standard error. To calculate the 95%
             confidence interval, use the range -1.96 to 1.96 times this value
         """
-        return (
-            self.standard_dg_prime(reaction)
-            + self.RT * reaction.dg_correction()
-        )
+        return self.standard_dg_prime(reaction) + self.RT * reaction.dg_correction()
 
     def standard_dg_prime_multi(
         self,
@@ -600,17 +590,14 @@ class ComponentContribution(object):
                 )
             except MissingDissociationConstantsException as e:
                 warnings.warn(
-                    f"Cannot calculate Legendre transform for reaction #{i}: "
-                    + str(e)
+                    f"Cannot calculate Legendre transform for reaction #{i}: " + str(e)
                 )
 
         if minimize_norm:
             # Calculate the residual matrix (X) that represents the DOFs that
             # are completely free (infinite uncertianty).
             _preprocessor = self.predictor.preprocess
-            X = _preprocessor.get_reaction_prediction_orthogonal_dof(
-                residual_reactions
-            )
+            X = _preprocessor.get_reaction_prediction_orthogonal_dof(residual_reactions)
 
             # Project the 'standard_dg_prime' onto the nullspace of the
             # residual matrix (using an orthogonal projection). This doesn't
@@ -621,9 +608,7 @@ class ComponentContribution(object):
 
         return standard_dg_prime, dg_uncertainty
 
-    def physiological_dg_prime(
-        self, reaction: PhasedReaction
-    ) -> ureg.Measurement:
+    def physiological_dg_prime(self, reaction: PhasedReaction) -> ureg.Measurement:
         """Calculate the dG'm of a single reaction.
 
         Assume all aqueous reactants are at 1 mM, gas reactants at 1 mbar and
@@ -646,9 +631,7 @@ class ComponentContribution(object):
             + self.RT * reaction.physiological_dg_correction()
         )
 
-    def ln_reversibility_index(
-        self, reaction: PhasedReaction
-    ) -> ureg.Measurement:
+    def ln_reversibility_index(self, reaction: PhasedReaction) -> ureg.Measurement:
         """Calculate the reversibility index (ln Gamma) of a single reaction.
 
         Returns
@@ -679,15 +662,11 @@ class ComponentContribution(object):
         if n_e is None:
             raise ValueError("reaction is not chemically balanced")
         if n_e == 0:
-            raise ValueError(
-                "this is not a half-reaction, " "electrons are balanced"
-            )
+            raise ValueError("this is not a half-reaction, " "electrons are balanced")
         standard_e_prime = -self.standard_dg_prime(reaction) / (n_e * FARADAY)
         return standard_e_prime.to("mV")
 
-    def physiological_e_prime(
-        self, reaction: PhasedReaction
-    ) -> ureg.Measurement:
+    def physiological_e_prime(self, reaction: PhasedReaction) -> ureg.Measurement:
         """Calculate the E'0 of a single half-reaction.
 
         Returns
@@ -702,9 +681,7 @@ class ComponentContribution(object):
         if n_e is None:
             raise ValueError("reaction is not chemically balanced")
         if n_e == 0:
-            raise ValueError(
-                "this is not a half-reaction, " "electrons are balanced"
-            )
+            raise ValueError("this is not a half-reaction, " "electrons are balanced")
         return -self.physiological_dg_prime(reaction) / (n_e * FARADAY)
 
     def e_prime(self, reaction: PhasedReaction) -> ureg.Measurement:
@@ -722,9 +699,7 @@ class ComponentContribution(object):
         if n_e is None:
             raise ValueError("reaction is not chemically balanced")
         if n_e == 0:
-            raise ValueError(
-                "this is not a half-reaction, " "electrons are balanced"
-            )
+            raise ValueError("this is not a half-reaction, " "electrons are balanced")
         return -self.dg_prime(reaction) / (n_e * FARADAY)
 
     def dg_analysis(self, reaction: PhasedReaction) -> List[Dict[str, object]]:
@@ -865,14 +840,97 @@ class ComponentContribution(object):
 
         return standard_dg + stored_dg_prime_inner + stored_dg_prime_outer
 
-    def create_stoichiometric_matrix(
-        self, reactions: Iterable[PhasedReaction]
+    @staticmethod
+    def parse_formula_side(s: str) -> Dict[str, float]:
+        """Parse one side of the reaction formula."""
+        if s.strip() == "null":
+            return {}
+        compound_bag = {}
+        for member in re.split(r"\s+\+\s+", s):
+            tokens = member.split(None, 1)
+            if len(tokens) == 0:
+                continue
+            if len(tokens) == 1:
+                amount = 1.0
+                compound = member
+            else:
+                try:
+                    amount = float(tokens[0])
+                except ValueError:
+                    raise ParseException(f"Non-specific reaction: {s}")
+                compound = tokens[1]
+            compound_bag[compound] = compound_bag.get(compound, 0.0) + amount
+        return compound_bag
+
+    @staticmethod
+    def parse_formula(formula: str) -> Dict[str, float]:
+        """Parse a two-sided reaction formula."""
+        tokens = []
+        arrow = None
+        for arrow in POSSIBLE_REACTION_ARROWS:
+            if formula.find(arrow) != -1:
+                tokens = formula.split(arrow, 2)
+                break
+
+        if len(tokens) < 2:
+            raise ParseException(
+                f"Reaction does not contain an allowed arrow sign ({arrow}): "
+                f" {formula}"
+            )
+
+        left = tokens[0].strip()
+        right = tokens[1].strip()
+
+        sparse_reaction = {}
+        left_dict = ComponentContribution.parse_formula_side(left)
+        right_dict = ComponentContribution.parse_formula_side(right)
+        for cid, count in left_dict.items():
+            sparse_reaction[cid] = sparse_reaction.get(cid, 0) - count
+
+        for cid, count in right_dict.items():
+            sparse_reaction[cid] = sparse_reaction.get(cid, 0) + count
+
+        # remove compounds that are balanced out in the reaction,
+        # i.e. their coefficient is 0
+        sparse_reaction = dict(filter(lambda x: x[1] != 0, sparse_reaction.items()))
+        return sparse_reaction
+
+    @staticmethod
+    def create_stoichiometric_matrix_from_reaction_formulas(
+        formulas: Iterable[str],
     ) -> pd.DataFrame:
         """Build a stoichiometric matrix.
 
         Parameters
         ----------
-        reactions : Iterable[Reaction]
+        formulas : Iterable[str]
+            String representations of the reactions.
+
+        Returns
+        -------
+        DataFrame
+            The stoichiometric matrix as a DataFrame whose indexes are the
+            compound IDs and its columns are the reaction IDs (in the same order
+            as the input).
+
+        """
+        return (
+            pd.DataFrame.from_dict(
+                [ComponentContribution.parse_formula(s) for s in formulas]
+            )
+            .fillna(0.0)
+            .transpose()
+        )
+
+    def create_stoichiometric_matrix_from_reaction_objects(
+        self,
+        reactions: Iterable[PhasedReaction],
+    ) -> pd.DataFrame:
+        """Build a stoichiometric matrix.
+
+        Parameters
+        ----------
+        reactions : Iterable[PhasedReaction]
             The collection of reactions to build a stoichiometric
             matrix from.
 
@@ -884,6 +942,7 @@ class ComponentContribution(object):
             the input).
 
         """
+
         return create_stoichiometric_matrix_from_reactions(
             reactions,
             self.ccache.is_proton,
