@@ -26,7 +26,7 @@
 # THE SOFTWARE.
 
 
-from typing import Dict, Iterable, TextIO, Tuple
+from typing import Dict, Iterable, TextIO, Tuple, Union
 
 import numpy as np
 import pandas as pd
@@ -50,46 +50,50 @@ class BaseBounds(object):
         """Return a (deep) copy of self."""
         raise NotImplementedError
 
-    def get_lower_bound(self, compound: Compound):
+    def get_lower_bound(self, compound: Union[str, Compound]):
         """Get the lower bound for this key.
 
         :param key: a compound
         """
         raise NotImplementedError
 
-    def get_upper_bound(self, compound: Compound):
+    def get_upper_bound(self, compound: Union[str, Compound]):
         """Get the upper bound for this key.
 
         :param key: a compound
         """
         raise NotImplementedError
 
-    def get_lower_bounds(self, compounds: Iterable[Compound]) -> Iterable[Q_]:
+    def get_lower_bounds(
+        self, compounds: Iterable[Union[str, Compound]]
+    ) -> Iterable[Q_]:
         """Get the bounds for a set of keys in order.
 
-        :param compounds: an iterable of Compounds
+        :param compounds: an iterable of Compounds or strings
         :return:  an iterable of the lower bounds
         """
         return map(self.get_lower_bound, compounds)
 
-    def get_upper_bounds(self, compounds: Iterable[Compound]) -> Iterable[Q_]:
+    def get_upper_bounds(
+        self, compounds: Iterable[Union[str, Compound]]
+    ) -> Iterable[Q_]:
         """Get the bounds for a set of keys in order.
 
-        :param compounds: an iterable of Compounds
+        :param compounds: an iterable of Compounds or strings
         :return:  an iterable of the upper bounds
         """
         return map(self.get_upper_bound, compounds)
 
-    def get_bound_tuple(self, compound: Compound) -> Tuple[Q_, Q_]:
+    def get_bound_tuple(self, compound: Union[str, Compound]) -> Tuple[Q_, Q_]:
         """Get both upper and lower bounds for this key.
 
-        :param compound: a Compound object
+        :param compound: a Compound object or string
         :return: a 2-tuple (lower bound, upper bound)
         """
         return self.get_lower_bound(compound), self.get_upper_bound(compound)
 
     def get_bounds(
-        self, compounds: Iterable[Compound]
+        self, compounds: Iterable[Union[str, Compound]]
     ) -> Tuple[Iterable[Q_], Iterable[Q_]]:
         """Get the bounds for a set of compounds.
 
@@ -111,11 +115,11 @@ class BaseBounds(object):
         return np.log((b / standard_concentration).m_as(""))
 
     def get_ln_bounds(
-        self, compounds: Iterable[Compound]
+        self, compounds: Iterable[Union[str, Compound]]
     ) -> Tuple[Iterable[float], Iterable[float]]:
         """Get the log-bounds for a set of compounds.
 
-        :param compounds: an iterable of Compounds
+        :param compounds: an iterable of Compounds or strings
         :return: a 2-tuple (log lower bounds, log upper bounds)
         """
         lbs, ubs = self.get_bounds(compounds)
@@ -123,33 +127,32 @@ class BaseBounds(object):
         return map(self.conc2ln_conc, lbs), map(self.conc2ln_conc, ubs)
 
     def get_ln_lower_bounds(
-        self, compounds: Iterable[Compound]
+        self, compounds: Iterable[Union[str, Compound]]
     ) -> Iterable[float]:
         """Get the log lower bounds for a set of compounds.
 
-        :param compounds: an iterable of Compounds
+        :param compounds: an iterable of Compounds or strings
         :return: an iterable of log lower bounds
         """
         lbs = self.get_lower_bounds(compounds)
         return map(self.conc2ln_conc, lbs)
 
     def get_ln_upper_bounds(
-        self, compounds: Iterable[Compound]
+        self, compounds: Iterable[Union[str, Compound]]
     ) -> Iterable[float]:
         """Get the log upper bounds for a set of compounds.
 
-        :param compounds: an iterable of Compounds
+        :param compounds: an iterable of Compounds or strings
         :return: an iterable of log upper bounds
         """
         ubs = self.get_upper_bounds(compounds)
         return map(self.conc2ln_conc, ubs)
 
     @ureg.check(None, None, "[concentration]", "[concentration]")
-    def set_bounds(self, compound: Compound, lb: Q_, ub: Q_) -> None:
+    def set_bounds(self, compound: Union[str, Compound], lb: Q_, ub: Q_) -> None:
         """Set bounds for a specific key.
 
-        :param key: a string representation of a KEGG compound ID,
-        i.e. C00001 for water
+        :param key: a Compounds or string
         :param lb: the lower bound value
         :param ub: the upper bound value
         """
@@ -168,8 +171,8 @@ class Bounds(BaseBounds):
     @ureg.check(None, None, None, "[concentration]", "[concentration]")
     def __init__(
         self,
-        lower_bounds: Dict[Compound, Q_] = None,
-        upper_bounds: Dict[Compound, Q_] = None,
+        lower_bounds: Dict[Union[str, Compound], Q_] = None,
+        upper_bounds: Dict[Union[str, Compound], Q_] = None,
         default_lb: Q_ = default_conc_lb,
         default_ub: Q_ = default_conc_ub,
     ) -> None:
@@ -214,16 +217,15 @@ class Bounds(BaseBounds):
         default_ub : Q_
             the default upper bound
         """
-        lbs: Dict[Compound, Q_] = dict()
-        ubs: Dict[Compound, Q_] = dict()
+        lbs: Dict[Union[str, Compound], Q_] = dict()
+        ubs: Dict[Union[str, Compound], Q_] = dict()
         bounds_df = pd.read_csv(f)
 
         for row in bounds_df.itertuples():
             compound = comp_contrib.get_compound(row.compound_id)
             if compound is None:
                 raise ValueError(
-                    f"Cannot find this compound accession: "
-                    f"{row.compound_id}"
+                    f"Cannot find this compound accession: " f"{row.compound_id}"
                 )
             lbs[compound] = row.lb * ureg.molar
             ubs[compound] = row.ub * ureg.molar
@@ -264,11 +266,11 @@ class Bounds(BaseBounds):
             self.default_ub,
         )
 
-    def get_lower_bound(self, compound: Compound) -> Q_:
+    def get_lower_bound(self, compound: Union[str, Compound]) -> Q_:
         """Get the lower bound for this compound."""
         return self.lower_bounds.get(compound, self.default_lb)
 
-    def get_upper_bound(self, compound: Compound) -> Q_:
+    def get_upper_bound(self, compound: Union[str, Compound]) -> Q_:
         """Get the upper bound for this compound."""
         return self.upper_bounds.get(compound, self.default_ub)
 
