@@ -105,12 +105,18 @@ class StoichiometricModel(object):
         # we keep the index to H2O handy, since we often need to remove
         # water from the list of compounds
         compounds = list(self.compounds)
-        self.idx_water = compounds.index(self.water)
-        # make sure the bounds on Water are (1, 1)
-        self._bounds.set_bounds(compounds[self.idx_water], Q_(1.0, "M"), Q_(1.0, "M"))
 
-        # TODO: properly deal with H+ (like water,
-        # they should have no effect on the results)
+        if self.water in compounds:
+            self.idx_water = compounds.index(self.water)
+            # make sure the bounds on Water are (1, 1)
+            self._bounds.set_bounds(
+                compounds[self.idx_water], Q_(1.0, "M"), Q_(1.0, "M")
+            )
+        else:
+            self.idx_water = -1
+
+        # TODO: properly deal with H+ (like water, they should have no
+        # effect on the results)
 
         self.Nc, self.Nr = self.S.shape
 
