@@ -27,11 +27,11 @@
 
 import re
 from collections import namedtuple
-from typing import Tuple
+from typing import Optional, Tuple
 
 import numpy as np
-from equilibrator_cache import PROTON_INCHI_KEY, Compound
-from equilibrator_cache.thermodynamic_constants import R, _legendre_transform
+from equilibrator_cache import Compound
+from equilibrator_cache.thermodynamic_constants import FARADAY, R, _legendre_transform
 
 from . import Q_, ureg
 
@@ -40,22 +40,19 @@ AQUEOUS_PHASE_NAME = "aqueous"
 GAS_PHASE_NAME = "gas"
 LIQUID_PHASE_NAME = "liquid"
 SOLID_PHASE_NAME = "solid"
+REDOX_PHASE_NAME = "redox"
 
 PhaseInfo = namedtuple(
     "phase_info",
-    "shorthand "
-    "standard_abundance "
-    "physiolical_abundance "
-    "dimensionality",
+    "shorthand " "standard_abundance " "physiolical_abundance " "dimensionality",
 )
 
 PHASE_INFO_DICT = {
-    AQUEOUS_PHASE_NAME: PhaseInfo(
-        "(aq)", Q_("1 M"), Q_("1 mM"), "[concentration]"
-    ),
+    AQUEOUS_PHASE_NAME: PhaseInfo("(aq)", Q_("1 M"), Q_("1 mM"), "[concentration]"),
     GAS_PHASE_NAME: PhaseInfo("(g)", Q_("1 bar"), Q_("1 mbar"), "[pressure]"),
     SOLID_PHASE_NAME: PhaseInfo("(s)", None, None, None),
     LIQUID_PHASE_NAME: PhaseInfo("(l)", None, None, None),
+    REDOX_PHASE_NAME: PhaseInfo("(redox)", Q_("0 V"), Q_("0 mV"), "[energy]/[charge]"),
 }
 
 # dictionary of compounds that can be non aqueous. The values are tuples of
@@ -97,6 +94,14 @@ PHASED_COMPOUND_DICT = {
     ),  # sulfur
 }
 
+CARBONATE_INCHIS = set(
+    [
+        "InChI=1S/CH2O3/c2-1(3)4/h(H2,2,3,4)",
+        "InChI=1S/CH2O3/c2-1(3)4/h(H2,2,3,4)/p-1",
+        "InChI=1S/CH2O3/c2-1(3)4/h(H2,2,3,4)/p-2",
+    ]
+)
+
 
 class Condition(object):
     """A class for defining the conditions of a compound.
@@ -171,6 +176,13 @@ class Condition(object):
         """
         if abundance is None:
             return
+        try:
+            abundance = Q_(float(abundance.magnitude), abundance.units)
+        except ValueError:
+            raise ValueError(
+                f"Cannot convert the given abundance to a proper Quantity: "
+                f"{abundance}"
+            )
 
         assert (
             self.standard_abundance is not None
@@ -205,7 +217,11 @@ class Condition(object):
 class PhasedCompound(object):
     """A class that combines a equilibrator_api Compound and a Condition."""
 
-    def __init__(self, compound: Compound, condition: Condition = None):
+    def __init__(
+        self,
+        compound: Compound,
+        condition: Condition = None,
+    ):
         """Create a new PhasedCompound object."""
         self.compound = compound
         self.condition = condition or Condition(self.possible_phases[0])
@@ -222,6 +238,16 @@ class PhasedCompound(object):
         else:
             return Condition(AQUEOUS_PHASE_NAME)
 
+    @property
+    def atom_bag(self) -> dict:
+        """Get the compound's atom bag."""
+        return self.compound.atom_bag or {}
+
+    @property
+    def smiles(self) -> str:
+        """Get the compound's InChI."""
+        return self.compound.smiles
+
     @property
     def inchi(self) -> str:
         """Get the compound's InChI."""
@@ -232,11 +258,6 @@ class PhasedCompound(object):
         """Get the compound's InChIKey."""
         return self.compound.inchi_key
 
-    @property
-    def is_proton(self) -> bool:
-        """Return True if this compound is a proton."""
-        return self.compound.inchi_key == PROTON_INCHI_KEY
-
     @property
     def id(self) -> int:
         """Get the compound's equilibrator internal ID."""
@@ -247,11 +268,6 @@ class PhasedCompound(object):
         """Get the chemical formula."""
         return self.compound.formula
 
-    @property
-    def html_formula(self) -> str:
-        """Get the chemical formula."""
-        return re.sub(r"(\d+)", r"<sub>\1</sub>", self.compound.formula)
-
     @property
     def mass(self) -> float:
         """Get the chemical molecular mass."""
@@ -262,11 +278,15 @@ class PhasedCompound(object):
         """Get the phase."""
         return self.condition.phase
 
+    @property
+    def html_formula(self) -> str:
+        """Get the chemical formula."""
+        return re.sub(r"(\d+)", r"<sub>\1</sub>", self.formula)
+
     @property
     def phase_shorthand(self) -> str:
         """Get the phase shorthand (i.e. 'l' for liquid)."""
-        if self.inchi == "InChI=1S/CH2O3/c2-1(3)4/h(H2,2,3,4)/p-1":
-            # TODO: compare the InChI to HCO3- without the proton layer.
+        if self.inchi in CARBONATE_INCHIS:
             # for CO2(total) there should be no indication of the phase,
             # since it is actually a combination of gas and aqueous.
             return ""
@@ -303,29 +323,16 @@ class PhasedCompound(object):
         :param abundance: the new abundance in the correct units
         :return:
         """
-        assert not self.is_proton, (
-            "Cannot directly change the relative abundance of H+. Use the pH "
-            "parameter instead for adjusting this value."
-        )
-
         self.condition.abundance = abundance
 
     @property
     def ln_abundance(self) -> float:
         """Return the log of the abundance (for thermodynamic calculations)."""
-        if self.is_proton:
-            # Since pH is an environmental parameter, we do not use the
-            # concentration of protons for energy calculations
-            return 0.0
         return self.condition.ln_abundance
 
     @property
     def ln_physiological_abundance(self) -> float:
         """Return the log of the default physiological abundance."""
-        if self.is_proton:
-            # Since pH is an environmental parameter, we do not use the
-            # concentration of protons for energy calculations
-            return 0.0
         return self.condition.ln_physiological_abundance
 
     @property
@@ -420,3 +427,97 @@ class PhasedCompound(object):
             "ln_abundance": self.ln_abundance,
             "microspecies": ms_list,
         }
+
+
+class Proton(PhasedCompound):
+    """A class specifically for protons."""
+
+    def __init__(self, compound: Compound):
+        """Create a RedoxCarrier object."""
+        super(Proton, self).__init__(
+            compound,
+            condition=Condition(AQUEOUS_PHASE_NAME, None),
+        )
+
+    @property
+    def abundance(self) -> ureg.Quantity:
+        """Get the abundance."""
+        return self.condition.standard_abundance
+
+    @abundance.setter
+    def abundance(self, abundance: ureg.Quantity) -> None:
+        """Change the phase of a specific compound.
+
+        :param abundance: the new abundance in the correct units
+        :return:
+        """
+        raise AssertionError(
+            "Cannot directly change the relative abundance of H+. Use the pH "
+            "parameter instead for adjusting this value."
+        )
+
+    @property
+    def ln_physiological_abundance(self) -> float:
+        """Return the log of the default physiological abundance."""
+        # Since pH is an environmental parameter, we do not use the
+        # concentration of protons for energy calculations
+        return 0.0
+
+    @property
+    def ln_abundance(self) -> float:
+        """Return the log of the abundance (for thermodynamic calculations)."""
+        # Since pH is an environmental parameter, we do not use the
+        # concentration of protons for energy calculations
+        return 0.0
+
+
+class RedoxCarrier(PhasedCompound):
+    """A class specifically for redox carriers (with a given potential)."""
+
+    def __init__(
+        self,
+        compound: Compound,
+        potential: Optional[ureg.Quantity] = None,
+    ):
+        """Create a RedoxCarrier object."""
+        potential = potential or Q_(0.0, "V")
+        assert potential.check("[energy]/[charge]")
+        super(RedoxCarrier, self).__init__(
+            compound,
+            condition=Condition(REDOX_PHASE_NAME, potential),
+        )
+
+    @ureg.check(None, None, "[concentration]", "[temperature]", "")
+    def get_stored_standard_dgf_prime(
+        self,
+        p_h: ureg.Quantity,
+        ionic_strength: ureg.Quantity,
+        temperature: ureg.Quantity,
+        p_mg: ureg.Quantity,
+    ) -> ureg.Quantity:
+        """Get the standard formation ΔG'."""
+        return self.get_stored_standard_dgf()
+
+    def get_stored_standard_dgf(self) -> ureg.Quantity:
+        """Get the standard formation ΔG."""
+        return -FARADAY * self.abundance
+
+    @property
+    def atom_bag(self) -> dict:
+        """Get the compound's atom bag."""
+        return {"e-": 1}
+
+    @property
+    def ln_abundance(self) -> float:
+        """Return the log of the abundance (for thermodynamic calculations)."""
+        return 0.0
+
+    @property
+    def ln_physiological_abundance(self) -> float:
+        """Return the log of the default physiological abundance."""
+        return 0.0
+
+    @property
+    def is_physiological(self) -> bool:
+        """Check if the abundance is physiological."""
+        return True
